"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.validate = exports.entries = exports.map = exports.get = exports.pointer = exports.value = exports.nil = undefined;

var _bluebird = require("bluebird");

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _jsonStableStringify = require("json-stable-stringify");

var _jsonStableStringify2 = _interopRequireDefault(_jsonStableStringify);

var _textEncoding = require("text-encoding");

var _jsonReference = require("@hyperjump/json-reference");

var JsonReference = _interopRequireWildcard(_jsonReference);

var _validationResult = require("./validation-result");

var ValidationResult = _interopRequireWildcard(_validationResult);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var nil = exports.nil = JsonReference.nil;
var value = exports.value = JsonReference.value;
var pointer = exports.pointer = JsonReference.pointer;
var get = exports.get = JsonReference.get;
var map = exports.map = JsonReference.map;
var entries = exports.entries = JsonReference.entries;

var _validate = function () {
  var _ref = (0, _bluebird.coroutine)( /*#__PURE__*/regeneratorRuntime.mark(function _callee(schema, memo) {
    var validatorList;
    return regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            if (memo[schema.url]) {
              _context.next = 6;
              break;
            }

            memo[schema.url] = function (_value) {
              return false;
            };

            _context.next = 4;
            return entries(schema).filter(function (_ref2) {
              var _ref3 = _slicedToArray(_ref2, 2),
                  keyword = _ref3[0],
                  _ = _ref3[1];

              return keyword in validators;
            }).map(function (_ref4) {
              var _ref5 = _slicedToArray(_ref4, 2),
                  keyword = _ref5[0],
                  keywordSchema = _ref5[1];

              return validators[keyword](keywordSchema, memo);
            });

          case 4:
            validatorList = _context.sent;


            memo[schema.url] = function (value) {
              return validatorList.map(function (validator) {
                return validator(value);
              });
            };

          case 6:
            return _context.abrupt("return", function (value) {
              return memo[schema.url](value);
            });

          case 7:
          case "end":
            return _context.stop();
        }
      }
    }, _callee, undefined);
  }));

  return function _validate(_x, _x2) {
    return _ref.apply(this, arguments);
  };
}();

var isObject = function isObject(value) {
  return (typeof value === "undefined" ? "undefined" : _typeof(value)) === "object" && !Array.isArray(value) && value !== null;
};

var isType = {
  "null": function _null(value) {
    return value === null;
  },
  "boolean": function boolean(value) {
    return typeof value === "boolean";
  },
  "object": isObject,
  "array": function array(value) {
    return Array.isArray(value);
  },
  "number": function number(value) {
    return typeof value === "number";
  },
  "string": function string(value) {
    return typeof value === "string";
  }
};

var validators = {
  "const": function _const(schema) {
    var ptr = pointer(schema);
    var expectedValue = value(schema);
    return function (value) {
      return [ptr, (0, _jsonStableStringify2.default)(value) === (0, _jsonStableStringify2.default)(expectedValue)];
    };
  },
  "type": function type(schema) {
    var ptr = pointer(schema);
    var type = value(schema);
    return function (value) {
      return [ptr, isType[type](value)];
    };
  },
  "multipleOf": function multipleOf(schema) {
    var ptr = pointer(schema);
    var multipleOf = value(schema);

    return function (value) {
      if (typeof value !== "number") {
        return [ptr, true];
      }

      var remainder = value % multipleOf;
      return [ptr, remainder === 0 || multipleOf - remainder < Number.EPSILON || remainder < Number.EPSILON];
    };
  },
  "maximum": function maximum(schema) {
    var ptr = pointer(schema);
    var maximum = value(schema);
    return function (value) {
      return [ptr, typeof value !== "number" || value <= maximum];
    };
  },
  "exclusiveMaximum": function exclusiveMaximum(schema) {
    var ptr = pointer(schema);
    var exclusiveMaximum = value(schema);
    return function (value) {
      return [ptr, typeof value !== "number" || value < exclusiveMaximum];
    };
  },
  "minimum": function minimum(schema) {
    var ptr = pointer(schema);
    var minimum = value(schema);
    return function (value) {
      return [ptr, typeof value !== "number" || value >= minimum];
    };
  },
  "exclusiveMinimum": function exclusiveMinimum(schema) {
    var ptr = pointer(schema);
    var exclusiveMinimum = value(schema);
    return function (value) {
      return [ptr, typeof value !== "number" || value > exclusiveMinimum];
    };
  },
  "maxLength": function maxLength(schema) {
    var ptr = pointer(schema);
    var length = value(schema);
    return function (value) {
      if (typeof value !== "string") {
        return [ptr, true];
      }

      var str = new _textEncoding.TextEncoder("utf-8").encode(value);
      return [ptr, str.length <= length];
    };
  },
  "minLength": function minLength(schema) {
    var ptr = pointer(schema);
    var length = value(schema);
    return function (value) {
      if (typeof value !== "string") {
        return [ptr, true];
      }

      var str = new _textEncoding.TextEncoder("utf-8").encode(value);
      return [ptr, str.length >= length];
    };
  },
  "pattern": function pattern(schema) {
    var ptr = pointer(schema);
    var pattern = new RegExp(value(schema));
    return function (value) {
      return [ptr, typeof value !== "string" || pattern.test(value)];
    };
  },
  "items": function () {
    var _ref6 = (0, _bluebird.coroutine)( /*#__PURE__*/regeneratorRuntime.mark(function _callee2(schema, memo) {
      var ptr, validateItem;
      return regeneratorRuntime.wrap(function _callee2$(_context2) {
        while (1) {
          switch (_context2.prev = _context2.next) {
            case 0:
              ptr = pointer(schema);
              _context2.next = 3;
              return _validate(schema, memo);

            case 3:
              validateItem = _context2.sent;
              return _context2.abrupt("return", function (value) {
                return [ptr, !Array.isArray(value) || value.every(function (item) {
                  return ValidationResult.isValid(validateItem(item));
                })];
              });

            case 5:
            case "end":
              return _context2.stop();
          }
        }
      }, _callee2, undefined);
    }));

    return function items(_x3, _x4) {
      return _ref6.apply(this, arguments);
    };
  }(),
  "tupleItems": function () {
    var _ref7 = (0, _bluebird.coroutine)( /*#__PURE__*/regeneratorRuntime.mark(function _callee3(schema, memo) {
      var ptr, tupleItems;
      return regeneratorRuntime.wrap(function _callee3$(_context3) {
        while (1) {
          switch (_context3.prev = _context3.next) {
            case 0:
              ptr = pointer(schema);
              _context3.next = 3;
              return map(function (itemSchema) {
                return _validate(itemSchema, memo);
              }, schema);

            case 3:
              tupleItems = _context3.sent;
              return _context3.abrupt("return", function (value) {
                return [ptr, !Array.isArray(value) || value.every(function (item, ndx) {
                  return !(ndx in tupleItems) || ValidationResult.isValid(tupleItems[ndx](item));
                })];
              });

            case 5:
            case "end":
              return _context3.stop();
          }
        }
      }, _callee3, undefined);
    }));

    return function tupleItems(_x5, _x6) {
      return _ref7.apply(this, arguments);
    };
  }(),
  "maxItems": function maxItems(schema) {
    var ptr = pointer(schema);
    var maxItems = value(schema);
    return function (value) {
      return [ptr, !Array.isArray(value) || value.length <= maxItems];
    };
  },
  "minItems": function minItems(schema) {
    var ptr = pointer(schema);
    var minItems = value(schema);
    return function (value) {
      return [ptr, !Array.isArray(value) || value.length >= minItems];
    };
  },
  "uniqueItems": function uniqueItems(schema) {
    var ptr = pointer(schema);
    var uniqueItems = value(schema);
    return function (value) {
      if (!Array.isArray(value) || uniqueItems === false) {
        return [ptr, true];
      }

      var normalizedItems = value.map(_jsonStableStringify2.default);
      return [ptr, new Set(normalizedItems).size === normalizedItems.length];
    };
  },
  "properties": function () {
    var _ref8 = (0, _bluebird.coroutine)( /*#__PURE__*/regeneratorRuntime.mark(function _callee5(schema, memo) {
      var ptr, propertyValidators;
      return regeneratorRuntime.wrap(function _callee5$(_context5) {
        while (1) {
          switch (_context5.prev = _context5.next) {
            case 0:
              ptr = pointer(schema);
              _context5.t0 = Map;
              _context5.next = 4;
              return entries(schema).map(function () {
                var _ref9 = (0, _bluebird.coroutine)( /*#__PURE__*/regeneratorRuntime.mark(function _callee4(_ref10) {
                  var _ref11 = _slicedToArray(_ref10, 2),
                      propertyName = _ref11[0],
                      propertySchema = _ref11[1];

                  return regeneratorRuntime.wrap(function _callee4$(_context4) {
                    while (1) {
                      switch (_context4.prev = _context4.next) {
                        case 0:
                          _context4.t0 = propertyName;
                          _context4.next = 3;
                          return _validate(propertySchema, memo);

                        case 3:
                          _context4.t1 = _context4.sent;
                          return _context4.abrupt("return", [_context4.t0, _context4.t1]);

                        case 5:
                        case "end":
                          return _context4.stop();
                      }
                    }
                  }, _callee4, undefined);
                }));

                return function (_x9) {
                  return _ref9.apply(this, arguments);
                };
              }());

            case 4:
              _context5.t1 = _context5.sent;
              propertyValidators = new _context5.t0(_context5.t1);
              return _context5.abrupt("return", function (value) {
                return [ptr, !isObject(value) || Object.keys(value).filter(function (propertyName) {
                  return propertyValidators.has(propertyName);
                }).every(function (propertyName) {
                  var validator = propertyValidators.get(propertyName);
                  var result = validator(value[propertyName]);
                  return ValidationResult.isValid(result);
                })];
              });

            case 7:
            case "end":
              return _context5.stop();
          }
        }
      }, _callee5, undefined);
    }));

    return function properties(_x7, _x8) {
      return _ref8.apply(this, arguments);
    };
  }(),
  "patternProperties": function () {
    var _ref12 = (0, _bluebird.coroutine)( /*#__PURE__*/regeneratorRuntime.mark(function _callee7(schema, memo) {
      var ptr, validators;
      return regeneratorRuntime.wrap(function _callee7$(_context7) {
        while (1) {
          switch (_context7.prev = _context7.next) {
            case 0:
              ptr = pointer(schema);
              _context7.next = 3;
              return entries(schema).map(function () {
                var _ref13 = (0, _bluebird.coroutine)( /*#__PURE__*/regeneratorRuntime.mark(function _callee6(_ref14) {
                  var _ref15 = _slicedToArray(_ref14, 2),
                      propertyPattern = _ref15[0],
                      propertySchema = _ref15[1];

                  return regeneratorRuntime.wrap(function _callee6$(_context6) {
                    while (1) {
                      switch (_context6.prev = _context6.next) {
                        case 0:
                          _context6.t0 = new RegExp(propertyPattern);
                          _context6.next = 3;
                          return _validate(propertySchema, memo);

                        case 3:
                          _context6.t1 = _context6.sent;
                          return _context6.abrupt("return", [_context6.t0, _context6.t1]);

                        case 5:
                        case "end":
                          return _context6.stop();
                      }
                    }
                  }, _callee6, undefined);
                }));

                return function (_x12) {
                  return _ref13.apply(this, arguments);
                };
              }());

            case 3:
              validators = _context7.sent;
              return _context7.abrupt("return", function (value) {
                if (!isObject(value)) {
                  return [ptr, true];
                }

                var propertyNames = Object.keys(value);
                return [ptr, validators.every(function (_ref16) {
                  var _ref17 = _slicedToArray(_ref16, 2),
                      pattern = _ref17[0],
                      validateProperty = _ref17[1];

                  return propertyNames.filter(function (propertyName) {
                    return pattern.test(propertyName);
                  }).every(function (propertyName) {
                    return ValidationResult.isValid(validateProperty(value[propertyName]));
                  });
                })];
              });

            case 5:
            case "end":
              return _context7.stop();
          }
        }
      }, _callee7, undefined);
    }));

    return function patternProperties(_x10, _x11) {
      return _ref12.apply(this, arguments);
    };
  }(),
  "propertyNames": function () {
    var _ref18 = (0, _bluebird.coroutine)( /*#__PURE__*/regeneratorRuntime.mark(function _callee8(schema, memo) {
      var ptr, validateName;
      return regeneratorRuntime.wrap(function _callee8$(_context8) {
        while (1) {
          switch (_context8.prev = _context8.next) {
            case 0:
              ptr = pointer(schema);
              _context8.next = 3;
              return _validate(schema, memo);

            case 3:
              validateName = _context8.sent;
              return _context8.abrupt("return", function (value) {
                return [ptr, !isObject(value) || Object.keys(value).every(function (name) {
                  return ValidationResult.isValid(validateName(name));
                })];
              });

            case 5:
            case "end":
              return _context8.stop();
          }
        }
      }, _callee8, undefined);
    }));

    return function propertyNames(_x13, _x14) {
      return _ref18.apply(this, arguments);
    };
  }(),
  "maxProperties": function maxProperties(schema) {
    var ptr = pointer(schema);
    var maxProperties = value(schema);
    return function (value) {
      return [ptr, !isObject(value) || Object.keys(value).length <= maxProperties];
    };
  },
  "minProperties": function minProperties(schema) {
    var ptr = pointer(schema);
    var minProperties = value(schema);
    return function (value) {
      return [ptr, !isObject(value) || Object.keys(value).length >= minProperties];
    };
  },
  "required": function required(schema) {
    var ptr = pointer(schema);
    var required = value(schema);
    return function (value) {
      return [ptr, !isObject(value) || required.every(function (propertyName) {
        return propertyName in value;
      })];
    };
  },
  "allOf": function () {
    var _ref19 = (0, _bluebird.coroutine)( /*#__PURE__*/regeneratorRuntime.mark(function _callee9(schema, memo) {
      var ptr, allOf;
      return regeneratorRuntime.wrap(function _callee9$(_context9) {
        while (1) {
          switch (_context9.prev = _context9.next) {
            case 0:
              ptr = pointer(schema);
              _context9.next = 3;
              return map(function (subSchema) {
                return _validate(subSchema, memo);
              }, schema);

            case 3:
              allOf = _context9.sent;
              return _context9.abrupt("return", function (value) {
                return [ptr, allOf.every(function (validateSchema) {
                  return ValidationResult.isValid(validateSchema(value));
                })];
              });

            case 5:
            case "end":
              return _context9.stop();
          }
        }
      }, _callee9, undefined);
    }));

    return function allOf(_x15, _x16) {
      return _ref19.apply(this, arguments);
    };
  }(),
  "anyOf": function () {
    var _ref20 = (0, _bluebird.coroutine)( /*#__PURE__*/regeneratorRuntime.mark(function _callee10(schema, memo) {
      var ptr, anyOf;
      return regeneratorRuntime.wrap(function _callee10$(_context10) {
        while (1) {
          switch (_context10.prev = _context10.next) {
            case 0:
              ptr = pointer(schema);
              _context10.next = 3;
              return map(function (subSchema) {
                return _validate(subSchema, memo);
              }, schema);

            case 3:
              anyOf = _context10.sent;
              return _context10.abrupt("return", function (value) {
                return [ptr, anyOf.some(function (validateSchema) {
                  return ValidationResult.isValid(validateSchema(value));
                })];
              });

            case 5:
            case "end":
              return _context10.stop();
          }
        }
      }, _callee10, undefined);
    }));

    return function anyOf(_x17, _x18) {
      return _ref20.apply(this, arguments);
    };
  }(),
  "oneOf": function () {
    var _ref21 = (0, _bluebird.coroutine)( /*#__PURE__*/regeneratorRuntime.mark(function _callee11(schema, memo) {
      var ptr, oneOf;
      return regeneratorRuntime.wrap(function _callee11$(_context11) {
        while (1) {
          switch (_context11.prev = _context11.next) {
            case 0:
              ptr = pointer(schema);
              _context11.next = 3;
              return map(function (subSchema) {
                return _validate(subSchema, memo);
              }, schema);

            case 3:
              oneOf = _context11.sent;
              return _context11.abrupt("return", function (value) {
                return [ptr, oneOf.reduce(function (acc, validateSchema) {
                  var isValid = ValidationResult.isValid(validateSchema(value));
                  return acc ? !isValid : isValid;
                }, false)];
              });

            case 5:
            case "end":
              return _context11.stop();
          }
        }
      }, _callee11, undefined);
    }));

    return function oneOf(_x19, _x20) {
      return _ref21.apply(this, arguments);
    };
  }(),
  "not": function () {
    var _ref22 = (0, _bluebird.coroutine)( /*#__PURE__*/regeneratorRuntime.mark(function _callee12(schema, memo) {
      var ptr, validateSchema;
      return regeneratorRuntime.wrap(function _callee12$(_context12) {
        while (1) {
          switch (_context12.prev = _context12.next) {
            case 0:
              ptr = pointer(schema);
              _context12.next = 3;
              return _validate(schema, memo);

            case 3:
              validateSchema = _context12.sent;
              return _context12.abrupt("return", function (schema) {
                return [ptr, !ValidationResult.isValid(validateSchema(schema))];
              });

            case 5:
            case "end":
              return _context12.stop();
          }
        }
      }, _callee12, undefined);
    }));

    return function not(_x21, _x22) {
      return _ref22.apply(this, arguments);
    };
  }()
};

var validate = exports.validate = function validate(schema) {
  return _validate(schema, {});
};